# emacs: -*- mode: python; py-indent-offset: 4; tab-width: 4; indent-tabs-mode: nil -*-
# ex: set sts=4 ts=4 sw=4 noet:
# ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
#
#   See COPYING file distributed along with the datalad package for the
#   copyright and license terms.
#
# ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##

import logging

from os import linesep

from ... import __version__
from ...dochelpers import exc_str
from ...version import __version__
from ..external_versions import ExternalVersions, LooseVersion
from ..exceptions import CommandError
from ..exceptions import OutdatedExternalDependency, MissingExternalDependency
from ...support.annexrepo import AnnexRepo
from ...tests.utils import (
    with_tempfile,
    create_tree,
    swallow_logs,
)

from mock import patch
from nose.tools import (
    assert_true, assert_false,
    assert_equal, assert_greater_equal, assert_greater,
    assert_raises, assert_in
)
from nose import SkipTest
from six import PY3

if PY3:
    # just to ease testing
    def cmp(a, b):
        return (a > b) - (a < b)


def test_external_versions_basic():
    ev = ExternalVersions()
    our_module = 'datalad'
    assert_equal(ev.versions, {})
    assert_equal(ev[our_module], __version__)
    # and it could be compared
    assert_greater_equal(ev[our_module], __version__)
    assert_greater(ev[our_module], '0.1')
    assert_equal(list(ev.keys()), [our_module])
    assert_true(our_module in ev)
    assert_false('unknown' in ev)

    # all are LooseVersions now
    assert_true(isinstance(ev[our_module], LooseVersion))
    version_str = __version__
    assert_equal(ev.dumps(), "Versions: %s=%s" % (our_module, version_str))

    # For non-existing one we get None
    assert_equal(ev['custom__nonexisting'], None)
    # and nothing gets added to _versions for nonexisting
    assert_equal(set(ev.versions.keys()), {our_module})

    # but if it is a module without version, we get it set to UNKNOWN
    assert_equal(ev['os'], ev.UNKNOWN)
    # And get a record on that inside
    assert_equal(ev.versions.get('os'), ev.UNKNOWN)
    # And that thing is "True", i.e. present
    assert(ev['os'])
    # but not comparable with anything besides itself (was above)
    assert_raises(TypeError, cmp, ev['os'], '0')
    assert_raises(TypeError, assert_greater, ev['os'], '0')

    return
    # Code below is from original duecredit, and we don't care about
    # testing this one
    # And we can get versions based on modules themselves
    from datalad.tests import mod
    assert_equal(ev[mod], mod.__version__)

    # Check that we can get a copy of the versions
    versions_dict = ev.versions
    versions_dict[our_module] = "0.0.1"
    assert_equal(versions_dict[our_module], "0.0.1")
    assert_equal(ev[our_module], __version__)


def test_external_version_contains():
    ev = ExternalVersions()
    assert_true("datalad" in ev)
    assert_false("does not exist" in ev)


def test_external_versions_unknown():
    assert_equal(str(ExternalVersions.UNKNOWN), 'UNKNOWN')


def _test_external(ev, modname):
    try:
        exec("import %s" % modname, globals(), locals())
    except ImportError:
        raise SkipTest("External %s not present" % modname)
    except Exception as e:
        raise SkipTest("External %s fails to import: %s" % (modname, exc_str(e)))
    assert (ev[modname] is not ev.UNKNOWN)
    assert_greater(ev[modname], '0.0.1')
    assert_greater('1000000.0', ev[modname])  # unlikely in our lifetimes


def test_external_versions_popular_packages():
    ev = ExternalVersions()

    for modname in ('scipy', 'numpy', 'mvpa2', 'sklearn', 'statsmodels', 'pandas',
                    'matplotlib', 'psychopy'):
        yield _test_external, ev, modname

    # more of a smoke test
    assert_false(linesep in ev.dumps())
    assert_true(ev.dumps(indent=True).endswith(linesep))


@with_tempfile(mkdir=True)
def test_external_versions_rogue_module(topd):
    ev = ExternalVersions()
    # if module throws some other non-ImportError exception upon import
    # we must not crash, but issue a warning
    modname = 'verycustomrogue__'
    create_tree(topd, {modname + '.py': 'raise Exception("pickaboo")'})
    with patch('sys.path', [topd]), \
        swallow_logs(new_level=logging.WARNING) as cml:
        assert ev[modname] is None
        assert_true(ev.dumps(indent=True).endswith(linesep))
        assert_in('pickaboo', cml.out)


def test_custom_versions():
    ev = ExternalVersions()
    assert(ev['cmd:annex'] > '6.20160101')  # annex must be present and recentish
    assert_equal(set(ev.versions.keys()), {'cmd:annex'})
    # some older git version don't support files to be passed to
    # `commit` call under some conditions and this will lead to diverse
    # errors
    assert(ev['cmd:git'] > '2.0')  # git must be present and recentish
    assert(isinstance(ev['cmd:git'], LooseVersion))
    assert_equal(set(ev.versions.keys()), {'cmd:annex', 'cmd:git'})

    # and there is also a version of system-wide installed git, which might
    # differ from cmd:git but should be at least good old 1.7
    assert(ev['cmd:system-git'] > '1.7')

    ev.CUSTOM = {'bogus': lambda: 1 / 0}
    assert_equal(ev['bogus'], None)
    assert_equal(set(ev.versions), {'cmd:annex', 'cmd:git', 'cmd:system-git'})


def test_ancient_annex():

    class _runner(object):
        def run(self, cmd):
            if '--raw' in cmd:
                raise CommandError
            return "git-annex version: 0.1", ""

    ev = ExternalVersions()
    with patch('datalad.support.external_versions._runner', _runner()):
        assert_equal(ev['cmd:annex'], '0.1')


def _test_annex_version_comparison(v, cmp_):
    class _runner(object):
        def run(self, cmd):
            return v, ""

    ev = ExternalVersions()
    with patch('datalad.support.external_versions._runner', _runner()), \
         patch('datalad.support.annexrepo.external_versions',
               ExternalVersions()):
        ev['cmd:annex'] < AnnexRepo.GIT_ANNEX_MIN_VERSION
        if cmp_ in (1, 0):
            AnnexRepo._check_git_annex_version()
            if cmp_ == 0:
                assert_equal(AnnexRepo.git_annex_version, v)
        elif cmp == -1:
            with assert_raises(OutdatedExternalDependency):
                ev.check('cmd:annex', min_version=AnnexRepo.GIT_ANNEX_MIN_VERSION)
            with assert_raises(OutdatedExternalDependency):
                AnnexRepo._check_git_annex_version()


def test_annex_version_comparison():
    # see https://github.com/datalad/datalad/issues/1128
    for cmp_, base in [(-1, '6.2011'), (1, "2100.0")]:
        # there could be differing versions of a version
        #   release, snapshot, neurodebian build of a snapshot
        for v in base, base + '-g0a34f08', base + '+gitg9f179ae-1~ndall+1':
            # they all must be comparable to our specification of min version
            yield _test_annex_version_comparison, v, cmp_
    yield _test_annex_version_comparison, str(AnnexRepo.GIT_ANNEX_MIN_VERSION), 0


def _test_list_tuple(thing):
    version = ExternalVersions._deduce_version(thing)
    assert_greater(version, '0.0.1')
    assert_greater('0.2', version)
    assert_equal('0.1', version)
    assert_equal(version, '0.1')


def test_list_tuple():

    class thing_with_tuple_version:
        __version__ = (0, 1)

    class thing_with_list_version:
        __version__ = [0, 1]

    for v in thing_with_list_version, thing_with_tuple_version, '0.1', (0, 1), [0, 1]:
        yield _test_list_tuple, v


def test_system_ssh_version():
    ev = ExternalVersions()
    assert ev['cmd:system-ssh']  # usually we have some available at boxes we test

    for s, v in [
        ('OpenSSH_7.4p1 Debian-6, OpenSSL 1.0.2k  26 Jan 2017', '7.4p1'),
    ]:
        ev = ExternalVersions()
        # TODO: figure out leaner way
        class _runner(object):
            def run(self, cmd, expect_fail, expect_stderr):
                return "", s
        with patch('datalad.support.external_versions._runner', _runner()):
            assert_equal(ev['cmd:system-ssh'], v)


def test_humanize():
    # doesn't provide __version__
    assert ExternalVersions()['humanize']


def test_check():
    ev = ExternalVersions()
    # should be all good
    ev.check('datalad')
    ev.check('datalad', min_version=__version__)

    with assert_raises(MissingExternalDependency):
        ev.check('dataladkukaracha')
    with assert_raises(MissingExternalDependency) as cme:
        ev.check('dataladkukaracha', min_version="buga", msg="duga")

    assert_in("duga", str(cme.exception))

    with assert_raises(OutdatedExternalDependency):
        ev.check('datalad', min_version="10000000")  # we will never get there!


# emacs: -*- mode: python; py-indent-offset: 4; tab-width: 4; indent-tabs-mode: nil -*-
# ex: set sts=4 ts=4 sw=4 noet:
# ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
#
#   See COPYING file distributed along with the datalad package for the
#   copyright and license terms.
#
# ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
"""Simple wrappers to get uniform JSON input and output
"""


import io
import codecs
from six import PY2
from os.path import (
    dirname,
    exists,
    lexists,
)
from os import makedirs
import os
import os.path as op

# wrapped below
from simplejson import load as jsonload
from simplejson import dump as jsondump
# simply mirrored for now
from simplejson import loads as json_loads
from simplejson import JSONDecodeError


# produce relatively compact, but also diff-friendly format
json_dump_kwargs = dict(
    indent=0,
    separators=(',', ':\n'),
    sort_keys=True,
    ensure_ascii=False,
    encoding='utf-8', )

# achieve minimal representation, but still deterministic
compressed_json_dump_kwargs = dict(
    json_dump_kwargs,
    indent=None,
    separators=(',', ':'))


# Let's just reuse top level one for now
from ..log import lgr
from ..dochelpers import exc_str


def dump(obj, fname, compressed=False):
    """Dump a JSON-serializable objects into a file

    Parameters
    ----------
    obj : object
      Structure to serialize.
    fname : str
      Name of the file to dump into.
    compressed : bool
      Flag whether to use LZMA compression for file content.
    """

    _open = LZMAFile if compressed else io.open

    indir = dirname(fname)
    if not exists(indir):
        makedirs(indir)
    if lexists(fname):
        os.unlink(fname)
    with _open(fname, 'wb') as f:
        return dump2fileobj(
            obj,
            f,
            **(compressed_json_dump_kwargs if compressed else json_dump_kwargs)
        )


def dump2fileobj(obj, fileobj, **kwargs):
    """Dump a JSON-serializable objects into a file-like

    Parameters
    ----------
    obj : object
      Structure to serialize.
    fileobj : file
      Writeable file-like object to dump into.
    **kwargs
      Keyword arguments to be passed on to simplejson.dump()
    """
    return jsondump(
        obj,
        codecs.getwriter('utf-8')(fileobj),
        **kwargs)


def LZMAFile(*args, **kwargs):
    """A little decorator to overcome a bug in lzma

    A unique to yoh and some others bug with pyliblzma
    calling dir() helps to avoid AttributeError __exit__
    see https://bugs.launchpad.net/pyliblzma/+bug/1219296
    """
    from .lzma import lzma
    lzmafile = lzma.LZMAFile(*args, **kwargs)
    dir(lzmafile)
    return lzmafile


def dump2stream(obj, fname, compressed=False):

    _open = LZMAFile if compressed else open

    indir = dirname(fname)

    if op.lexists(fname):
        os.remove(fname)
    elif indir and not exists(indir):
        makedirs(indir)
    with _open(fname, mode='wb') as f:
        jwriter = codecs.getwriter('utf-8')(f)
        for o in obj:
            jsondump(o, jwriter, **compressed_json_dump_kwargs)
            f.write(b'\n')


def dump2xzstream(obj, fname):
    dump2stream(obj, fname, compressed=True)


def load_stream(fname, compressed=None):
    _open = LZMAFile \
        if compressed or compressed is None and fname.endswith('.xz') \
        else io.open

    with _open(fname, mode='rb') as f:
        jreader = codecs.getreader('utf-8')(f)
        for line in jreader:
            yield loads(line)


def load_xzstream(fname):
    for o in load_stream(fname, compressed=True):
        yield o


def loads(s, *args, **kwargs):
    """Helper to log actual value which failed to be parsed"""
    try:
        return json_loads(s, *args, **kwargs)
    except:
        lgr.error(
            "Failed to load content from %r with args=%r kwargs=%r"
            % (s, args, kwargs)
        )
        raise


def load(fname, fixup=True, compressed=None, **kw):
    """Load JSON from a file, possibly fixing it up if initial load attempt fails

    Parameters
    ----------
    fixup : bool
      In case of failed load, apply a set of fixups with hope to resolve issues
      in JSON
    compressed : bool or None
      Flag whether to treat the file as XZ compressed. If None, this decision
      is made automatically based on the presence of a '.xz' extension in the
      filename
    **kw
      Passed into the load (and loads after fixups) function
    """
    _open = LZMAFile \
        if compressed or compressed is None and fname.endswith('.xz') \
        else io.open

    with _open(fname, 'rb') as f:
        try:
            jreader = codecs.getreader('utf-8')(f)
            return jsonload(jreader, **kw)
        except JSONDecodeError as exc:
            if not fixup:
                raise
            lgr.warning("Failed to decode content in %s: %s. Trying few tricks", fname, exc_str(exc))

            # Load entire content and replace common "abusers" which break JSON
            # comprehension but in general
            # are Ok
            with _open(fname,'rb') as f:
                s_orig = s = codecs.getreader('utf-8')(f).read()

            for o, r in {
                u"\xa0": " ",  # non-breaking space
            }.items():
                s = s.replace(o, r)

            if s == s_orig:
                # we have done nothing, so just reraise previous exception
                raise
            return loads(s, **kw)



# emacs: -*- mode: python; py-indent-offset: 4; tab-width: 4; indent-tabs-mode: nil; coding: utf-8 -*-
# ex: set sts=4 ts=4 sw=4 noet:
# ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
#
#   See COPYING file distributed along with the datalad package for the
#   copyright and license terms.
#
# ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
"""Primarily a smoke test for ls

"""

__docformat__ = 'restructuredtext'

import sys

from glob import glob
from collections import Counter
from mock import patch

from datalad.support.gitrepo import GitRepo
from datalad.support.annexrepo import AnnexRepo
from datalad.distribution.dataset import Dataset
from ...api import ls
from ...utils import swallow_outputs, chpwd
from ...tests.utils import assert_equal
from ...tests.utils import assert_in
from ...tests.utils import use_cassette
from ...tests.utils import with_tempfile
from ...tests.utils import skip_if_no_network
from ..ls import LsFormatter
from os.path import relpath
from os import mkdir

from datalad.downloaders.tests.utils import get_test_providers


@skip_if_no_network
@use_cassette('test_ls_s3')
def test_ls_s3():
    url = 's3://datalad-test0-versioned/'
    with swallow_outputs():
        # just to skip if no credentials
        get_test_providers(url)

    with swallow_outputs() as cmo:
        res = ls(url)
        assert_equal(len(res), 17)  # all the entries
        counts = Counter(map(lambda x: x.__class__.__name__, res))
        assert_equal(counts, {'Key': 14, 'DeleteMarker': 3})
        assert_in('Bucket info:', cmo.out)
test_ls_s3.tags = ['network']


@with_tempfile
def test_ls_repos(toppath):
    # smoke test pretty much
    GitRepo(toppath + '1', create=True)
    AnnexRepo(toppath + '2', create=True)
    repos = glob(toppath + '*')
    # now make that sibling directory from which we will ls later
    mkdir(toppath)
    def _test(*args_):
        #print args_
        for args in args_:
            for recursive in [False, True]:
                # in both cases shouldn't fail
                with swallow_outputs() as cmo:
                    ls(args, recursive=recursive)
                    assert_equal(len(cmo.out.rstrip().split('\n')), len(args))
                    assert_in('[annex]', cmo.out)
                    assert_in('[git]', cmo.out)
                    assert_in('master', cmo.out)
                    if "bogus" in args:
                        assert_in('unknown', cmo.out)

    _test(repos, repos + ["/some/bogus/file"])
    # check from within a sibling directory with relative paths
    with chpwd(toppath):
        _test([relpath(x, toppath) for x in repos])


@with_tempfile
def test_ls_uninstalled(path):
    ds = Dataset(path)
    ds.create()
    ds.create('sub')
    ds.uninstall('sub', check=False)
    with swallow_outputs() as cmo:
        ls([path], recursive=True)
        assert_in('not installed', cmo.out)


@with_tempfile
def test_ls_noarg(toppath):
    # smoke test pretty much
    AnnexRepo(toppath, create=True)

    # this test is pointless for now and until ls() actually returns
    # something
    with swallow_outputs():
        ls_out = ls(toppath)
        with chpwd(toppath):
            assert_equal(ls_out, ls([]))
            assert_equal(ls_out, ls('.'))


def test_ls_formatter():
    # we will use unicode symbols only when sys.stdio supports UTF-8
    for sysioenc, OK, tty in [(None, "OK", True),
                              ('ascii', 'OK', True),
                              ('UTF-8', u"✓", True),
                              ('UTF-8', "OK", False)]:

        # we cannot overload sys.stdout.encoding
        class fake_stdout(object):
            encoding = sysioenc
            def write(self, *args):
                pass

            def isatty(self):
                return tty

        with patch.object(sys, 'stdout', fake_stdout()):
            formatter = LsFormatter()
            assert_equal(formatter.OK, OK)
            assert_in(OK, formatter.convert_field(True, 'X'))



# emacs: -*- mode: python-mode; py-indent-offset: 4; tab-width: 4; indent-tabs-mode: nil -*-
# ex: set sts=4 ts=4 sw=4 noet:
# ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
#
#   See COPYING file distributed along with the datalad package for the
#   copyright and license terms.
#
# ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
"""Test command call protocols

Note: DryRunProtocol and NullProtocol are already (kind of) tested within
      test_cmd.py
"""

import os
from os.path import normpath
from nose.tools import ok_, eq_, assert_is, assert_equal, assert_greater, \
    assert_raises, assert_in, assert_is_instance, assert_true, assert_false

from ..support.protocol import DryRunProtocol, DryRunExternalsProtocol, \
    NullProtocol, ExecutionTimeProtocol, ExecutionTimeExternalsProtocol, \
    ProtocolInterface
from ..support.gitrepo import GitRepo
from ..cmd import Runner
from .utils import with_tempfile
from .utils import swallow_logs


@with_tempfile
def test_protocol_commons(protocol_file):

    for protocol_class in [DryRunProtocol, DryRunExternalsProtocol,
                           ExecutionTimeProtocol,
                           ExecutionTimeExternalsProtocol, NullProtocol]:
        protocol = protocol_class()
        assert_is_instance(protocol, ProtocolInterface)
        assert_equal(len(protocol), 0)

        protocol.add_section(['some_command', 'some_option'],
                             Exception("Whatever exception"))
        protocol.add_section(['another_command'], None)
        assert_equal(len(protocol), 2 if protocol_class != NullProtocol else 0)

        # test iterable:
        assert_raises(AssertionError, assert_raises, TypeError, iter,
                      protocol)
        for section in protocol:
            assert_in('command', section)
        for item in range(len(protocol)):
            assert_is_instance(protocol.__getitem__(item), dict)

        # test __str__:
        str_ = str(protocol)

        # test write_to_file:
        protocol.write_to_file(protocol_file)
        read_str = ''
        with open(protocol_file, 'r') as f:
            for line in f.readlines():
                read_str += line
        assert_equal(str_, read_str)


@with_tempfile
@with_tempfile
def test_ExecutionTimeProtocol(path1, path2):

    timer_protocol = ExecutionTimeProtocol()
    runner = Runner(protocol=timer_protocol)

    # test external command:
    cmd = ['git', 'init']
    os.mkdir(path1)
    runner.run(cmd, cwd=path1)
    assert_equal(len(timer_protocol), 1, str(runner.protocol))
    assert_equal(cmd, timer_protocol[0]['command'])
    ok_(timer_protocol[0]['end'] >= timer_protocol[0]['start'])
    ok_(timer_protocol[0]['duration'] >= 0)
    assert_is(timer_protocol[0]['exception'], None)

    # now with exception, since path2 doesn't exist yet:
    try:
        with swallow_logs() as cml:
            runner.run(cmd, cwd=path2)
    except Exception as e:
        catched_exception = e
    finally:
        assert_equal(len(timer_protocol), 2)
        assert_equal(cmd, timer_protocol[1]['command'])
        ok_(timer_protocol[1]['end'] >= timer_protocol[1]['start'])
        ok_(timer_protocol[1]['duration'] >= 0)
        assert_is(timer_protocol[1]['exception'], catched_exception)

    # test callable:
    new_runner = Runner(cwd=path2, protocol=timer_protocol)
    new_runner(os.mkdir, path2)
    assert_equal(len(timer_protocol), 3)
    assert_in('mkdir', timer_protocol[2]['command'][0])

    # extract path from args and compare
    # note: simple string concatenation for comparison doesn't work
    # on windows due to path conversion taking place
    ok_(timer_protocol[2]['command'][1].startswith("args=('"))
    extracted_path = timer_protocol[2]['command'][1].split(',')[0][7:-1]
    assert_equal(normpath(extracted_path), normpath(path2))

    # kwargs needs to be in protocol, but order isn't relevant:
    ok_("kwargs={}" in timer_protocol[2]['command'][2])
    ok_(timer_protocol[2]['end'] >= timer_protocol[2]['start'])
    ok_(timer_protocol[2]['duration'] >= 0)


@with_tempfile
@with_tempfile
def test_ExecutionTimeExternalsProtocol(path1, path2):

    timer_protocol = ExecutionTimeExternalsProtocol()
    runner = Runner(protocol=timer_protocol)

    # test external command:
    cmd = ['git', 'init']
    os.mkdir(path1)
    runner.run(cmd, cwd=path1)
    assert_equal(len(timer_protocol), 1, str(runner.protocol))
    assert_equal(cmd, timer_protocol[0]['command'])
    ok_(timer_protocol[0]['end'] >= timer_protocol[0]['start'])
    ok_(timer_protocol[0]['duration'] >= 0)
    assert_is(timer_protocol[0]['exception'], None)

    # now with exception, since path2 doesn't exist yet:
    try:
        with swallow_logs() as cml:
            runner.run(cmd, cwd=path2)
    except Exception as e:
        catched_exception = e
    finally:
        assert_equal(len(timer_protocol), 2)
        assert_equal(cmd, timer_protocol[1]['command'])
        ok_(timer_protocol[1]['end'] >= timer_protocol[1]['start'])
        ok_(timer_protocol[1]['duration'] >= 0)
        assert_is(timer_protocol[1]['exception'], catched_exception)

    # test callable (no entry added):
    new_runner = Runner(cwd=path2, protocol=timer_protocol)
    new_runner(os.mkdir, path2)
    assert_true(os.path.exists(path2))
    assert_equal(len(timer_protocol), 2)


@with_tempfile
def test_DryRunProtocol(path):

    protocol = DryRunProtocol()
    runner = Runner(protocol=protocol, cwd=path)
    cmd = ['git', 'init']

    # path doesn't exist, so an actual run would raise Exception,
    # but a dry run wouldn't:
    with swallow_logs() as cml:
        assert_raises(AssertionError, assert_raises, Exception, runner.run, cmd)
    assert_equal(len(protocol), 1)

    # callable is also not executed, but recorded in the protocol:
    runner(os.mkdir, path)
    assert_false(os.path.exists(path))
    assert_false(os.path.exists(os.path.join(path, '.git')))
    assert_equal(len(protocol), 2)


@with_tempfile
def test_DryRunExternalsProtocol(path):

    protocol = DryRunExternalsProtocol()
    runner = Runner(protocol=protocol, cwd=path)
    cmd = ['git', 'init']

    # path doesn't exist, so an actual run would raise Exception,
    # but a dry run wouldn't:
    assert_raises(AssertionError, assert_raises, Exception, runner.run, cmd)
    assert_equal(len(protocol), 1)
    assert_false(os.path.exists(path))

    # callable is executed and not recorded in the protocol:
    runner(os.mkdir, path)
    assert_true(os.path.exists(path))
    assert_equal(len(protocol), 1)


#!/usr/bin/env python
"""
Check which test did not come back to initial directory and fail right away
when detected.

Could be used as easy as replacement of `python -m nose ...`, with
`python checkpwd.py --with-checkpwd ...`
"""

import os
from os import path as op
import logging
from nose.plugins.base import Plugin
from nose.util import src, tolist

log = logging.getLogger(__name__)


def getpwd():
    """Try to return a CWD without dereferencing possible symlinks

    If no PWD found in the env, output of getcwd() is returned
    """
    cwd = os.getcwd()
    try:
        env_pwd = os.environ['PWD']
        if op.realpath(env_pwd) != op.realpath(cwd):
            # uses os.chdir directly, pwd is not updated
            # could be an option to fail (tp not allow direct chdir)
            return cwd
        return env_pwd
    except KeyError:
        return cwd


class CheckPWD(Plugin):
    """
    Activate a coverage report using Ned Batchelder's coverage module.
    """
    name = 'checkpwd'

    def options(self, parser, env):
        """
        Add options to command line.
        """
        # throw_exception = True
        super(CheckPWD, self).options(parser, env)

    def configure(self, options, conf):
        """
        Configure plugin.
        """
        super(CheckPWD, self).configure(options, conf)
        self._pwd = getpwd()
        print("Initial PWD: %s" % self._pwd)

    def beforeTest(self, *args, **kwargs):
        """
        Begin recording coverage information.
        """
        assert getpwd() == self._pwd

    def afterTest(self, *args, **kwargs):
        """
        Stop recording coverage information.
        """
        pwd = getpwd()
        # print("Checking %s" % pwd)
        print("PWD: %s" % pwd)
        assert pwd == self._pwd, \
            "PWD original:%s  current: %s (after %s)" \
            % (self._pwd, pwd, args[0])


def test_ok():
    pass


def test_fail():
    os.chdir('/dev')


def test_fail_not_again():
    # will never reach here if test_fail fails
    pass


if __name__ == '__main__':
    import nose
    nose.main(addplugins=[CheckPWD()])